import { types } from '@babel/core';
import { format } from 'util';
import { getConfigExports } from "./utils/getConfigExports.js";
import { prepareConfig } from "./utils/prepareConfig.js";
var configsCache = new Map();
var exportsCache = new Map();

function getConfigs(modules) {
  if (configsCache.size === 0) {
    for (var config of prepareConfig(modules)) {
      configsCache.set(config.name, config);
    }
  }

  return configsCache;
}

function getExports(name, modules) {
  var pkgExports = exportsCache.get(name);

  if (!pkgExports) {
    var config = getConfigs(modules).get(name);

    if (!config) {
      pkgExports = new Map();
    } else {
      pkgExports = getConfigExports(config);
    }

    exportsCache.set(name, pkgExports);
  }

  return pkgExports;
}

export default function plugin() {
  return {
    visitor: {
      ImportDeclaration(declaration, _ref) {
        var {
          opts
        } = _ref;
        var {
          source,
          specifiers,
          importKind
        } = declaration.node;

        if (!opts || !opts.modules || !Array.isArray(opts.modules) || opts.modules.length === 0) {
          return;
        }

        if (importKind === 'type') {
          return;
        }

        if (specifiers.length === 0) {
          return;
        }

        var exports = getExports(source.value, opts.modules);

        if (exports.size === 0) {
          return;
        }

        var _loop = function _loop(specifier) {
          if (types.isImportNamespaceSpecifier(specifier)) {
            // eslint-disable-next-line no-console
            console.warn(format('babel-plugin-direct-import: Can not optimize `import * as %s from "%s"`.'.concat('\n', 'See plugin limitations https://git.io/vFDOO for more details.'), specifier.local.name, source.value));
            return "continue";
          }

          var moduleName = types.isImportDefaultSpecifier(specifier) ? 'default' : specifier.imported.name;
          var moduleExports = exports.get(moduleName);

          if (moduleExports) {
            declaration.node.specifiers = declaration.node.specifiers.filter(x => x !== specifier);
            declaration.insertBefore(types.importDeclaration([moduleExports.internal === '*' ? types.importNamespaceSpecifier(types.identifier(specifier.local.name)) : moduleExports.internal === 'default' ? types.importDefaultSpecifier(types.identifier(specifier.local.name)) : types.importSpecifier(types.identifier(specifier.local.name), types.identifier(moduleExports.external))], types.stringLiteral(moduleExports.source)));
          }
        };

        for (var specifier of specifiers) {
          var _ret = _loop(specifier);

          if (_ret === "continue") continue;
        }

        if (declaration.node.specifiers.length === 0) {
          declaration.remove();
        }
      }

    }
  };
}